#include <QPainter>
#include <QTextBlock>
#include "editor.h"
#include "selectedarea.h"


Editor::Editor(QWidget *parent) :
    QPlainTextEdit(parent)
{
    selectedArea = new SelectedArea(this);

    connect(this, SIGNAL(blockCountChanged(int)), SLOT(updateSelectedAreaWidth(int)));
    connect(this, SIGNAL(updateRequest(QRect,int)), SLOT(updateSelectedArea(QRect,int)));
    connect(this, SIGNAL(cursorPositionChanged()), SLOT(highlightCurrentLine()));

    updateSelectedAreaWidth(0);
    highlightCurrentLine();
}

int Editor::selectedAreaWidth()
{
    int digits = 1;
    int max = qMax(1, blockCount());

    while (max >= 10)
    {
        max /= 10;
        ++digits;
    }

    int space = 3 + fontMetrics().width(QLatin1Char('9')) * digits;
    return space;
}

void Editor::resizeEvent(QResizeEvent *e)
{
    QPlainTextEdit::resizeEvent(e);
    QRect cr = contentsRect();
    selectedArea->setGeometry(QRect(cr.left(), cr.top(), selectedAreaWidth(), cr.height()));
}

void Editor::updateSelectedAreaWidth(int /* nb_counter */)
{
    setViewportMargins(selectedAreaWidth(), 0, 0, 0);
}

void Editor::updateSelectedArea(const QRect &rect, int dy)
{
    if (dy)
        selectedArea->scroll(0, dy);
    else
        selectedArea->update(0, rect.y(), selectedArea->width(), rect.height());

    if (rect.contains(viewport()->rect()))
        updateSelectedAreaWidth(0);
}

void Editor::highlightCurrentLine()
{
    QList<QTextEdit::ExtraSelection> extraSelections;

    if (!isReadOnly())
    {
        QTextEdit::ExtraSelection selection;

        QColor lineColor = QColor(Qt::lightGray).lighter(110);

        selection.format.setBackground(lineColor);
        selection.format.setProperty(QTextFormat::FullWidthSelection, true);
        selection.cursor = textCursor();
        selection.cursor.clearSelection();
        extraSelections.append(selection);
    }
    setExtraSelections(extraSelections);
}

void Editor::SelectedAreaFill(QPaintEvent *event)
{
    QPainter painter(selectedArea);
    painter.fillRect(event->rect(), Qt::green);
    QTextBlock block = firstVisibleBlock();
    int blockNumber = block.blockNumber();
    int top = (int) blockBoundingGeometry(block).translated(contentOffset()).top();
    int bottom = top + (int) blockBoundingRect(block).height();

    while (block.isValid() && top <= event->rect().bottom())
    {
        if (block.isVisible() && bottom >= event->rect().top())
        {
            QString number = QString::number(blockNumber + 1);
            painter.setPen(Qt::black);
            painter.drawText(0, top,
                             selectedArea->width(),
                             fontMetrics().height(),
                             Qt::AlignRight, number);
        }

        block = block.next();
        top = bottom;
        bottom = top + (int) blockBoundingRect(block).height();
        ++blockNumber;
    }
}
