#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGridLayout>
#include <QTabWidget>
#include <QString>
#include <QMenu>
#include <QMessageBox>
#include <QErrorMessage>
#include <QFile>
#include <QFileDialog>
#include <QFileInfo>
#include <QPrinter>
#include <QDockWidget>
#include <QListWidget>
#include <QListWidgetItem>
#include <QTreeView>
#include <QDirModel>
#include <QModelIndex>
#include <QToolBar>
#include <QTabBar>
#include <QAction>
#include <QToolButton>
#include <QCloseEvent>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMimeData>
#include <QList>
#include <QUrl>

#include "selectedarea.h"
#include "editor.h"
#include "highlighter.h"
#include "menu.h"


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    Ui::MainWindow *ui;
    QTabWidget* tabs;
    Highlighter* highlighter;

    QDirModel* f_sys_model;
    QTreeView* f_sys_view;
    QListWidget* open_docsW;
    ExitMenu* submenu;

    QDockWidget* f_explorer;
    QDockWidget* open_docs;

    QVector<QDir> file_path_menu;
    QStringList file_name_menu;

    QAction* newFile;
    QAction* openFile;
    QAction* saveFile;
    QAction* saveAllFiles;

    unsigned int number;
    bool bug_t;

    void SetupTabWidget();
    void SetupMenuBar();
    void SetupToolBar();
    void SetupFileExplorer();
    void SetupOpenDocs();

protected:
    void dragEnterEvent(QDragEnterEvent* drag_event) override;
    void dropEvent(QDropEvent* drop_event) override;
    void closeEvent(QCloseEvent*) override;

private slots:
    void UpdateParam();
    void CreateFile();
    void OpenFile();
    void OpenFile(const QString&);
    void OpenFile(QModelIndex);
    void SaveFile();
    void SaveFileAs();
    void SaveAllFiles();
    void CloseFile();
    void CloseFile(int);
    void CloseAllFiles();
    void CloseWindow();
    void Cut();
    void Copy();
    void Paste();
    void SelectAll();
    void CLear();
    void ChangeTabIndexInList(int, int);
    void DeleteTabFromList(int);
    void UpdateCurrentIndex(int);
    void UpdateCurrentIndex(QListWidgetItem*);
    void UpdateCurrentIndexOnDelete(int);
    void UpdateContains(int, int);
};

#endif // MAINWINDOW_H
